<?php
// $Id: uprpc.admin.inc,v 1.1.4.4 2011/02/17 22:16:39 joachim Exp $
/**
 * @file uprpc.admin.inc
 * Contains admin menu callbacks.
 */

/**
 * Menu callback.
 */
function uprpc_settings() {
  $form = array();

  $form['connection'] = array(
    '#type' => 'fieldset',
    '#title' => t('Remote site connection'),
    '#collapsible' => TRUE,
  );
  
  $form['connection']['uprpc_client_connection'] = array(
    '#type' => 'select',
    '#title' => t('Connection'),
    '#description' => t('Select a Clients connection to use for the transport demo. You may create and edit connections on the <a href="!clients-url">Clients admin page</a>. UpRPC module adds further connection tests to your selected connection.', array('!clients-url' => url('admin/settings/clients/connections'))),
    '#default_value' => variable_get('uprpc_client_connection', 0),
    '#options' => clients_connections_select_options('drupal_services_5'),
    '#required' => TRUE,
  );

  $form['content'] = array(
    '#type' => 'fieldset',
    '#title' => t('Content to import'),
    '#collapsible' => TRUE,
  );
  
  $form['content']['node_types'] = array(
    '#type' => 'fieldset',
    '#title' => t('Node types'),
    '#collapsible' => TRUE,
    '#tree' => TRUE,
    '#theme' => 'uprpc_node_types',
    '#description' => t('Select each node type to import. Optionally, enter the machine name of the node type on the remote site if it is different.'),
  );
  
  $node_types = node_get_types();
  $uprpc_node_types = array_flip(variable_get('uprpc_node_types', array()));
  //dsm($uprpc_node_types);
  foreach ($node_types as $type_name => $type) {
    $form['content']['node_types'][$type_name]['checkbox'] = array(
      '#type' => 'checkbox',
      '#default_value' => isset($uprpc_node_types[$type_name]),
      //'#title' => t('Import @type', array('@type' => $type->name)),
    );
    $form['content']['node_types'][$type_name]['label'] = array(
      '#value' => $type_name . ' (' . check_plain($type->name) . ')',
    );
    if (isset($uprpc_node_types[$type_name]) && $type_name != $uprpc_node_types[$type_name]) {
      $default_remote_type = $uprpc_node_types[$type_name];
    }
    else {
      $default_remote_type = '';
    }
    $form['content']['node_types'][$type_name]['remote'] = array(
      '#type' => 'textfield',
      '#size' => 25,
      '#default_value' => $default_remote_type,
    );  
  }

  $form['content']['uprpc_vids'] = array(
    '#type' => 'textfield',
    '#title' => t('Vocabularies'),
    '#description' => t("Specify the IDs of vocabularies to import, as a comma-separated list. Example: '1,3,4'."),
    '#default_value' => variable_get('uprpc_vids', ''),
    '#size' => 60,
    '#maxlength' => 128,
  );
  
  $form['import_settings'] = array(
    '#type' => 'fieldset',
    '#title' => t('Import settings'),
    '#collapsible' => TRUE,
  );

  $form['import_settings']['uprpc_remote_mark'] = array(
    '#type' => 'checkbox',
    '#title' => t('Mark imported items on the remote site'),
    '#description' => t("If selected, each imported item will receive a flag on the remote site to mark it having been imported. Unselect this to have all items imported (and hence updated) every time the import is run."),
    '#default_value' => variable_get('uprpc_remote_mark', TRUE),
  );
  
  $form['import_settings']['uprpc_remote_filter'] = array(
    '#type' => 'radios',
    '#title' => t('Items to import'),
    '#description' => t("Which items to import, depending on whether they have been previously imported or updated on the remote site since the previous import."),
    '#options' => array(
      'new'     => t('Only items not yet imported'),
      'updated' => t('Items not yet imported or updated since the last import (only applies to nodes)'),
      'all'     => t('All items regardless of import status'),
    ),
    '#default_value' => variable_get('uprpc_remote_filter', 'new'),
  );  

  // Pass to system_settings_form rather than return it so we can add our submit handlers too.
  $form = system_settings_form($form);
  
  array_unshift($form['#submit'], 'uprpc_settings_form_node_types_submit');
  
  // TEMP
  //unset($form['#submit'][1]);
  
  //dsm($form);
  return $form;
}

/**
 * Theme the node type settings table.
 */
function theme_uprpc_node_types($form) {
  $rows = array();
  foreach (element_children($form) as $key) {
    $row = array();
    $row[] = drupal_render($form[$key]['checkbox']);
    $row[] = drupal_render($form[$key]['remote']);
    $row[] = '->';
    $row[] = drupal_render($form[$key]['label']);
    
    $rows[] = $row;
  }
  
  $headers = array(
    t('Import node type'),
    t('Remote type name machine name'),
    '',
    t('Local type'),
  );
  
  return theme('table', $headers, $rows);
}

/**
 * Extra submit handler for the settings form.
 */
function uprpc_settings_form_node_types_submit($form, &$form_state) {
  //dsm($form_state, 'fs');
  
  // We save node types settings as an array of types to import where each entry
  // is of the form REMOTE_TYPE => LOCAL_TYPE.
  foreach ($form_state['values']['node_types'] as $type => $values) {
    if ($values['checkbox']) {
      $remote_type  = empty($values['remote']) ? $type : $values['remote'];
      $local_type   = $type;
      $uprpc_node_types[$remote_type] = $local_type;
    }
  }
  
  variable_set('uprpc_node_types', $uprpc_node_types);
  
  // Unset the values so system_settings_form_submit() doesn't see them.
  unset($form_state['values']['node_types']);
}
