<?php

/**
 * @file uprpc.pages.inc
 * Contains the form for the import page and its submit handler.
 */

/**
 * Implementation of hook_uprpc_batch_operation_info().
 *
 * Define our base import operations.
 *
 * TODO: add spec for this in an api.php file.
 *
 * For now, in haste: this defines operations available in the batch selection.
 * 
 * @return
 *  An array of operations, keyed by callback.
 *  The callback is assumed to be in a file called MODULENAME.batch.inc -- 
 *  TODO -- add a 'file' key to each op.
 *  The array should contain:
 *    - 'title': the label of the operation.
 *    - 'weight'.
 *  TODO: impose dependencies?
 */
function uprpc_uprpc_batch_operation_info($op = 'batch_ops') {
  return array(
    'uprpc_batch_terms' => array(
      'title'    => t("Import terms"),
      'weight'  => -10, // should come before nodes.
    ),
  );
}

/**
 * Form API callback
 */
function uprpc_page_go($form_values) {
  module_load_include("inc", "uprpc");

  // Now check memory limit, execution time
  // TODO: change these to dsm().
  $max_execution_time = ini_get('max_execution_time');
  if ($max_execution_time < 600) {
    $warning['time'] = array('#value' => t("Max execution time is !time s. This is too short: consider at least 600s.", array("!time" => $max_execution_time)));
  }
  $connection_id = variable_get('uprpc_client_connection', 0);
  if (!$connection_id) {
    $warning['client'] = array('#value' => t("No client connection set up. Please check the settings."));
  }
  if ($warning) {
    return $warning;
  }

  // Operations
  $form['operations'] = array(
    '#type' => 'fieldset',
    '#title' => t('Operations'),
    '#description' => t('Select one or more import operations.'),
    '#tree' => TRUE,
  );
  
  // Invoke hook_uprpc_batch_operation_info() to get all import operations 
  // defined by both ourselves and other modules.
  $operations = module_invoke_all('uprpc_batch_operation_info');
  // Sort them by their weight.
  uasort($operations, '_user_sort');

  foreach ($operations as $operation_id => $data) {
    $form['operations'][$operation_id] = array(
      '#type' => 'checkbox',
      '#title' => $data['title'],
    );
  }

  // TODO: move this to a separate form; it makes no sense with the 'import' button.
  $form['reset'] = array(
    '#type' => 'checkbox',
    '#title' => 'Reset all <strong>NODE</strong> import-tracking metrics and start again',
  );

  $form['import'] = array(
    '#type' => 'submit',
    '#value' => 'Import',
  );
  return $form;
}

/**
 * Form submit callback
 */
function uprpc_page_go_submit(&$form, &$form_state) {
  if ($form_state['values']['reset']) {
    module_load_include("inc", "uprpc");
    uprpc_reset_import();
  }

  // Get a flat array of the checked operations.
  $operations = array_keys(array_filter($form_state['values']['operations']));

  module_load_include("inc", "uprpc", "uprpc.batch");
  uprpc_batch_go($operations);
}
